﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8
@EndOfDesignText@
Sub Class_Globals
	Private mAttribution	As String=""		'attribution string for credits
	Private mModule			As Object			'the calling module
	Private mEvent 			As String
	Private mInterval		As Long				'check interval
	
	Private tmrcheck		As Timer
	
	'remember last request to not overload the server
	Private lastCheck		As Long				'last check time
	Private lastLat			As Double
	Private lastLon			As Double
	Private laterLat		As Double
	Private laterLon		As Double
	
	'decode
	Private mAddress As Map			'full map with results
	
End Sub

'Initializes the object.
'pass the module and event in order to process the result in your project
'the event is in the format: event_Done if we get a result or event_Error for errors
Public Sub Initialize(Module As Object, Event As String)
	mAttribution=""				'will be set at the first useful result
	mAddress.Initialize
	mModule=Module
	mEvent=Event
	mInterval=1000
	lastCheck=0
	lastLon=0
	lastLat=0
	
	'
	tmrcheck.Initialize("TmrReverse",1000)
	
End Sub

'gets or sets the minimum interval in milliseconds between server requests
'the minimum interval time is 1000 milliseconds
public Sub setRequestInterval(Interval As Long)
	mInterval=Max(1000,Interval)

End Sub

public Sub getRequestInterval As Long
	Return mInterval
End Sub

'return the attribution (license) from the result
'it is valid only after the first request from server (if no errors occurred)
public Sub Attribution As String
	Return mAttribution
End Sub


'try the reverse geocoding later...
private Sub TryLater(Latitude As Double, Longitude As Double,Delay As Long)
	laterLat=Latitude
	laterLon=Longitude
	tmrcheck.Interval=Delay
	tmrcheck.Enabled=True
End Sub

Private Sub TmrReverse_Tick
	tmrcheck.Enabled=False
	ReverseGeocode(laterLat,laterLon)
End Sub



'Start the reverse Geocode task,
'pass the latitude and longitude of your point
Public Sub ReverseGeocode(Latitude As Double,Longitude As Double)
	'https://nominatim.openstreetmap.org/reverse.php?format=html
	'https://nominatim.openstreetmap.org/reverse.php?format=json&lat=12.847585&lon=12.608305&zoom=18
	'Stop the timer task if the timer is already running, this allow the reverse geocoding to start a new request
	If tmrcheck.Enabled Then 
		tmrcheck.Enabled=False
	End If
	
	'end the task if the interval is not ended or lat and lon are not changed
	If (DateTime.Now-lastCheck<mInterval) Then
		Dim sleepFor As Long=mInterval-(DateTime.Now-lastCheck)
		#if developer
			Log("ReverseGeocoding is sleeping..." & sleepFor & "ms") 
		#End If
		'try later
		If sleepFor > 0 Then 
			TryLater(Latitude,Longitude,sleepFor)
			Return
		End If
		
	End If
	
	If Latitude=lastLat And Longitude=lastLon Then 
		#if developer
		Log("ReverseGeocoding already done on this location...")
		#End If
		Return
	End If
	
	lastLat=Latitude
	lastLon=Longitude
	
	#if developer
	Log("ReverseGeocoding started...")
	#End If

	'try to get an approximate location
	Dim pageJob As HttpJob
	pageJob.Initialize("RevGeocode",Me)
	Dim page As String="https://nominatim.openstreetmap.org/reverse.php?format=json&lat=" & Latitude & _
	"&lon=" & Longitude & "&zoom=18"
	pageJob.Download(page)
			
	Wait For (pageJob) JobDone(pageJob As HttpJob)
	If pageJob.Success Then
		'got a result?
		Try
			Dim res As String=pageJob.GetString
			#if developer
				Log("ReverseGeocoding result: " & CRLF & res)
			#End If
			'ClearResult		'be careful, this will clear the last result
			Decode(res)
			If SubExists(mModule,mEvent & "_done") Then
				CallSubDelayed(mModule,mEvent & "_done")
			End If
		Catch
			Log(LastException)
			If SubExists(mModule,mEvent & "_error") Then
				CallSubDelayed(mModule,mEvent & "_error")
			End If
		End Try
	Else
		'no result
		'raise an event on error
		If SubExists(mModule,mEvent & "_error") Then
			CallSubDelayed(mModule,mEvent & "_error")
		End If
	End If
	
	pageJob.Release
	'last
	lastCheck=DateTime.Now

	
End Sub

'clear the result, use it before Decode if needed
'do not clear the attribution
private Sub ClearResult
	mAddress.Initialize
End Sub

'decode the result
private Sub Decode(jsonText As String)
	
	Dim parser As JSONParser
	parser.Initialize(jsonText)
	Dim root As Map = parser.NextObject
	'
	Dim osm_type As String  = root.Get("osm_type")
	Dim osm_id  As String	= root.Get("osm_id")
	Dim place_id As String 	= root.Get("place_id")
	Dim lon As String 		= root.Get("lon")
	Dim lat As String 		= root.Get("lat")
	mAttribution 			= root.Get("licence")				'use it for attribution in your project!
	
	Dim display_name As String  = root.Get("display_name")		'most important!
	
	'not used, it's the area around the given location
	Dim boundingbox As List = root.Get("boundingbox")
	For Each colboundingbox As String In boundingbox
		#if developer
			Log("Bounding box coords: " & colboundingbox)
		#End If
	Next
	'
	mAddress  = root.Get("address")
	mAddress.Put("boundingbox",boundingbox)		'bounding box LIST moved to the address map
	mAddress.Put("display_name",display_name)	'display_name (full address) STRING moved to the address map
	mAddress.Put("osm_type",osm_type)			'osm_type STRING moved to the address map
	mAddress.Put("osm_id",osm_id)				'osm_id   STRING moved to the address map
	mAddress.Put("place_id",place_id)			'place_id STRING moved to the address map
	mAddress.Put("lon",lon)						'lon STRING moved to the address map
	mAddress.Put("lat",lat)						'lat STRING moved to the address map
	
	
End Sub

'get a result from the address map
'pass the key to get it's content, pay attention at the returned object!
'Available keys (with fake example):
'country 		- STRING ex. Italia
'country_code 	- STRING ex. it 
'town			- STRING ex. Bologna
'road			- STRING ex. via Amerigo Vespucci
'postcode		- STRING ex. 40033
'house_number	- STRING ex. 12
'county			- STRING ex. Unione dei comuni Valli del Reno, Lavino e Samoggia
'state			- STRING ex. EMR
'display_name  	- STRING ex. 12, via Amerigo Vespucci, Bologna, Unione dei comuni Valli del Reno, Lavino e Samoggia, BO, EMR, 40033, Italia
'boundingbox	- LIST: a box around the given location 
'there are also some other keys, see the decode sub
public Sub getResult(Query As String) As Object
	If mAddress.ContainsKey(Query) Then
		Return mAddress.Get(Query)
	Else
		#if developer
		Log("Reverse geocoding: Query key not found")
		#End If
		Return ""
	End If
	
End Sub

'********************************************************************************************************************************************

'{"place_id":"12121212",
'"licence":"Data © OpenStreetMap contributors, ODbL 1.0. https:\/\/osm.org\/copyright",
'"osm_type":"node",
'"osm_id":"0000000000",
'"lat":"12.345678",
'"lon":"12.345678",
'"display_name":"12, Via xxx xxxxxx, xxxxxxx xx xxxxxx xxx, Unione dei comuni Valli del Reno, Lavino e Samoggia, BO, EMR, 40033, Italia",
'"address":{"house_number":"12",
'"road":"xxx xxx xxxxxx",
'"town":"xxxxxxxxxxx xx xxxx",
'"county":"Unione dei comuni Valli del Reno, Lavino e Samoggia",
'"state":"EMR",
'"postcode":"40033",
'"country":"Italia",
'"country_code":"it"},
'"boundingbox":["12.111111","12.489952","12.252184","12.252384"]}

'Dim parser As JSONParser
'parser.Initialize(<text>)
'Dim root As Map = parser.NextObject
'Dim osm_type As String = root.Get("osm_type")
'Dim osm_id As String = root.Get("osm_id")
'Dim licence As String = root.Get("licence")
'Dim boundingbox As List = root.Get("boundingbox")
'For Each colboundingbox As String In boundingbox
'Next
'Dim address As Map = root.Get("address")
'Dim country As String = address.Get("country")
'Dim country_code As String = address.Get("country_code")
'Dim town As String = address.Get("town")
'Dim road As String = address.Get("road")
'Dim county As String = address.Get("county")
'Dim postcode As String = address.Get("postcode")
'Dim house_number As String = address.Get("house_number")
'Dim state As String = address.Get("state")
'Dim lon As String = root.Get("lon")
'Dim display_name As String = root.Get("display_name")
'Dim place_id As String = root.Get("place_id")
'Dim lat As String = root.Get("lat")



